@@ -1,5 +1,8 @@
 from .exceptions import *
 from . import util
+from .util import Role, Env
+role = Role()
+env = Env()
 from . import token
 from .token import *
 from .core import *
